
Use 'make all' or install each module separately [e.g. 'make photocouplings']

Gnuplot can be called to generate plots at the end of each script using the optional '-plot' flag.

Plots polarization observables, KLL and ALL, for the photoproduction process γ p → J/ψ p with the inclusion of pentaquark states Pc(4380) (wide state) and Pc(4450) (narrow state) in the s-channel.

The two Pc states are of spin 3/2 or 5/2 and opposite parity.

http://cgl.soic.indiana.edu/jpac/PentaQ_JPsi.php#results

## PARAMETERS

### /background-inputs/
Contains best fit values for background parameters for each JP:
    [all values should be floats]
 * 1 - Pomeron parameter a_0 (unitless)
 * 2 - Pomeron parameter a' (in GeV^-2)
 * 3 - Pomeron parameter A_norm (unitless)
 * 4 - Pomeron parameter s_t (in GeV^2)
 * 5 - Pomeron parameter b_0 (in GeV^-2)
 * 6 - Branching ratio Upper Limit at 95% CL (unitless)
 * 7 - Mass of the P_c(4450) (in GeV)
 * 8 - Width of the P_c(4450) (in GeV)
 * 9 - Spin/parity of the P_c(4450): has to be 3.l/2.l or 5.l/2.l, the sign gives parity
 * 10 - Branching ratio Upper Limit at 95% CL (unitless)
 * 11 - Mass of the P_c(4380) (in GeV)
 * 12 - Width of the P_c(4380) (in GeV)
 * 13 - Spin/parity of the P_c(4800) opposite parity and spin from other state

### Defaults Values
 The Pc(4450) pentaquark state is better constrained and the BR at the 95% CL from our fits for each spin-parity combination (in corresponding background-input) is used by default. The spin-parity of the narrow Pc(4450) state is 3/2- unless specified otherwise. The BR of the Pc(4380) is 0 by default.

Observables are evaluated at forward angles (cos = 1) and at resonance energy (10 GeV) with equal photocouplings unless specified by arguments.

### Other Options
Any functions has optional flags '-s' to include experimental smearing in GeV (default no smearing), and '-b' to change the step size of the data points (default .01).
e.g.
```
 ./fixed-angle -c 0. -s .1 -b .01 -s .05 -plot
```
Plots at fixed angle theta = 90° with 100 MeV smearing and in energy bins of 50 MeV.


# FUNCTIONS

### ./fixed-angle
Plots Observables DXS, KLL, and ALL versus energy at specified angle.
###### Options
```
              -c [cosine of scattering angle]
              -j [J^P of Pc(4450) (e.g. -2.5 for 5/2^-)]
              -p_(n/w) [Photocoupling ratio for narrow/wide Pc state]
              -r_(n/w) [Branching Ratio of narrow/wide Pc State]
```

###### Output
              /output/fixed-angle.dat
              /output/plots/ en-kll.pdf, en-all.pdf, en-dxs.pdf

### ./fixed-energy
Plots Observables DXS, KLL, ALL versus cos(THETA_cm) at specified energy.
###### Options
```
               -e [energy in GeV]
               -j [J^P of Pc(4450) (e.g. -2.5 for 5/2^-)]
               -p_(n/w) [Photocoupling ratio] -s [experimental smearing (GeV)]
               -r_(n/w) [Branching Ratio of narrow/width Pc State]
```

###### Output
              /output/fixed-energy.dat
              /output/plots/ ang-kll.pdf, ang-all.pdf, ang-dxs.pdf

### ./highenergy
High Energy Behavior of Differential Cross-section for photoproduction Background
Plots DXS (nb GeV^-2) vs -t (GeV^2) at large s small momentum transfer.
###### Options
```
             -e [energy in TeV]
             -j [J^P of Pc(4450) (e.g. -2.5 for 5/2^-)]
             -r_(n/w) [Branching Ratio of Pc States]
```
###### Output
              /output/large-s.dat
              /output/plots/large-s-dxs.pdf

### ./photo-(narrow/wide)
Plots ALL and KLL vs Beam Energy (GeV) for three photocoupling scenarios of
  the narrow/wide Pc state with the other held fixed at specified angle.
A1/2 = 0 , A1/2 = A3/2, and A1/2 = 0
###### Options
```
             -c [cosine of angle]
             -j [J^P of resonance]
             -r_(n/w) [Branching Ratio of Pc States]
             -p [Photocouplings of the fixed Pc State]
```
###### Output            
              /output/photocouplings.dat
              /output/plots/ photocouplings-kll.pdf, photocouplings-all.pdf

### ./spin-parity
Plots ALL and KLL vs Beam Energy (GeV) for four spin-parity combinations at specified angle.
###### Options
```
              -c [cosine of angle]
              -p_(n/w) [photocoupling ratio]
```
###### Output
              /output/spin-parity.dat
              /output/plots/ spin-parity-kll.dat, spin-parity-all.dat

### ./2dplot
Plots ALL and KLL as heatmaps in Energy-angle space.
###### Options
```
              -j [spin-parity of Pc(4450)]
              -p_(n/w) [Photocoupling ratio]
              -r_(n/w) [Branching Ratio of Pc State]
```
###### Output
              /output/2d.dat
              /output/plots/ 2d-kll.pdf 2d-all.pdf

### ./mc2
Generates events weighted by amplitude.
###### Options
```
              -n [# of Events] -e [Energy (GeV)]
              -s [J^P of Pc(4450) (e.g. -2.5 for 5/2^-)]
              -p [Photocoupling ratio]
              -b [Branching Fraction]
              -x [Radiation Length of the radiator]
              -q [flag to activate the quick generation, only geometrical acceptance is considered]
              -saveall [flag to save all the events in the ROOT file, instead of the one in the acceptance only]
              -bins [number of bins in energy]
              -min [lower value of the energy region (GeV)]
              -max [upper value of the energy region (GeV)]
              -sm [smearing in energy (GeV)]

              Either -e (fixed photon energy) or -x (description of the radiator, for fixed electron energy of 11 GeV) can be used
              -bins, -min and -max can be used only with X0
              -sm can only be used with -x and -bins

```
###### Defaults
              n = 10,000  energy = 10. GeV
              s = -1.5   p = 0.7071  b = 0.02
###### Output
              /output/###.root
	      /txt/###.root
              (actual filenames depend on the input parameters)
	

## REFERENCES
Appropriate equation numbers in comments from references:

For Resonance Contribution:

[WINN] - D. Winney et al. (JPAC collaboration),
   "Double Polarization Observables in Pentaquark Photoproduction,"
   arXiv:1907.09393

http://inspirehep.net/record/1744761?ln=en

[BLIN] - A. N. Hiller Blin et al. (JPAC collaboration),
   "Studying the Pc(4450) resonance in J/ψ photoproduction off protons,"
    Phys. Rev. D 94, 034002 (2016)

http://inspirehep.net/record/1473609?ln=en

For Photoproduction Background:

[LESN] - L. Lesniak, A. P. Szczepaniak,
    "Theoretical Model of the Phi Meson Photoproduction Amplitudes,"
     Acta Phys.Polon.B34:3389-3400 (2003)

https://arxiv.org/abs/hep-ph/0304007

## CLEANING

Use 'make clean-exe' to clean all executables and 'make clean-out' to clear
everything in the output/plot folders. Or simply 'make clean' for both.

Use 'make spotless' to delete all executables, and the output/obj folders entirely.
