#include "amp.h"


void ObservablesParam(cld egam, cld cos, cld *input, cld *output)
{
		cld AmpFunc(cld egam, cld cos, cld *input, int i);
		void DSG20R(ld& xlow, ld& xhigh, ld& n, ld *xd, ld& np);
		void DRG20_complex(ld& xlow, ld& xhigh, ld& n, cld *fd_c, cld& int_result);

		cld DXSFunc(cld * amps);
		void KLL(cld * amps, cld * output);
		void ALL(cld * amps, cld * output);

		cld amps[24];
		cld norm;
		cld total;
		cld KLLND[2];
		cld ALLND[2];

		if (abs(real(input[16])) < 0.0001l) // No experimental smearing
		{
				for (int i = 0; i < 24; i++ )
				{
						amps[i] = AmpFunc(egam, cos, input, i);
				}

				norm = (6084.375l * alpha) / (pow(egam, 2.l) * pow(mpro, 2.l));
				total = DXSFunc(amps);
				KLL(amps, KLLND);
				ALL(amps,ALLND);

				output[0] = norm*total;
				output[1] = KLLND[0]/KLLND[1];
				output[2] = ALLND[0]/ALLND[1];
		}
		else
		{
				cld smear = input[16];
				int i;
				ld enerd[CONST1], np;
				ld enerlow  = real((pow(mjpsi, 2.l) + 2.l * mpro * mjpsi) / (2.l * mpro));// lower bound
				ld enerhigh = 22.l;                                        // higher bound
				ld n        = ACCSMEAR;                                    // accuracy

				cld numerator;
				cld denominator;
				cld numerator2;
				cld denominator2;

				DSG20R(enerlow, enerhigh, n, enerd, np);

				// cld den_valueDXS[CONST1];
				cld den_val_NUM[CONST1];
				cld den_val_DEN[CONST1];
				cld den_val_NUM2[CONST1];
				cld den_val_DEN2[CONST1];

				for (i = 0; i < int(np); i++)
				{
						for (int a = 0; a < 24; a++ )
						{
								amps[a] = AmpFunc(enerd[i], cos, input, a);
						}
						// norm = (6084.375l * alpha) / (pow(enerd[i], 2.l) * pow(mpro, 2.l));
						// total = DXSFunc(amps);
						KLL(amps, KLLND);
						ALL(amps,ALLND);

						// den_valueDXS[i] = norm * total * exp(-pow(egam - enerd[i], 2.l) / (2.l * pow(smear, 2.l))) / (sqrt(2.l * Pi) * smear);
						den_val_NUM[i] = KLLND[0] * exp(-pow(egam - enerd[i], 2.l) / (2.l * pow(smear, 2.l))) / (sqrt(2.l * Pi) * smear);
						den_val_DEN[i] = KLLND[1] * exp(-pow(egam - enerd[i], 2.l) / (2.l * pow(smear, 2.l))) / (sqrt(2.l * Pi) * smear);
						den_val_NUM2[i] = ALLND[0] * exp(-pow(egam - enerd[i], 2.l) / (2.l * pow(smear, 2.l))) / (sqrt(2.l * Pi) * smear);
						den_val_DEN2[i] = ALLND[1] * exp(-pow(egam - enerd[i], 2.l) / (2.l * pow(smear, 2.l))) / (sqrt(2.l * Pi) * smear);
				}

				// DRG20_complex(enerlow, enerhigh, n, den_valueDXS, output[0]);
				DRG20_complex(enerlow, enerhigh, n, den_val_NUM, numerator);
				DRG20_complex(enerlow, enerhigh, n, den_val_DEN, denominator);
				DRG20_complex(enerlow, enerhigh, n, den_val_DEN2, denominator2);
				DRG20_complex(enerlow, enerhigh, n, den_val_NUM2, numerator2);

				output[1] = numerator / denominator;
				output[2] = numerator2 / denominator2;
		}

		return;
}

/*=======================================================================================
 *  AUXILIARY FUNCTIONS FOR HELICITY AMPLITUDES
 *=========================================================================================*/

// Mandelstam variable, t, in terms of lab frame photon energy.
cld t_mand(cld egam, cld cos)
{
		cld result = (1.l / (2.l * egam + mpro)) * (pow(mjpsi, 2.l) * mpro - 2.l * mpro * pow(egam, 2.l) + egam * (pow(mjpsi, 2.l)
		                                                                                                           + cos * sqrt(mpro * (2.l * egam + mpro)) * sqrt(4.l * pow(mpro, 2.l) * pow(egam, 2.l) - 4.l * mpro * (egam + mpro) * pow(mjpsi, 2.l) + pow(mjpsi, 4.l))
		                                                                                                           / sqrt(mpro * (2.l * egam + mpro))));
		return result;
}

cld kallen(cld x, cld y, cld z)
{
		return pow(x,2.l)+pow(y,2.l)+pow(z,2.l)-2.l*x*y-2.l*x*z - 2.l*y*z;
}
double kallen(double x, double y, double z)
{
		return x*x + y*y + z*z - 2.*x*y- 2.*x*z - 2.*y*z;
}
//Scattering angle of the recoil proton in the t-channel LAB frame
cld get_cosLAB(cld egam, cld cos)
{
		cld t = t_mand(egam, cos);
		cld s = mpro * (2.l * egam + mpro);
		cld mp2=pow(mpro,2.l);
		cld mjp2=pow(mjpsi,2.l);
		cld u = 2.l*mp2+mjp2-s-t;

		cld result = ((t-mp2)*(2.l*mp2-s)+2.l*mp2*(u-mp2))/sqrt(kallen(t,0.l,mp2)*kallen(s,mp2,mp2));
		return result;
}

// 24 Helicity Amplitudes are needed to describe jpsi production. Only 12 ( lambda_gamma = +1)
// is used for the background
// Helicity configuration for Photon [0], Incoming Proton [1], Jpsi [2], Outgoing Proton [3]
void get_helicity(int i, cld *output)
{
		cld helicities [24][4] = {
				{  1.l, -1.l,  1.l, -1.l },
				{  1.l, -1.l,  1.l,  1.l },
				{  1.l, -1.l,  0.l, -1.l },
				{  1.l, -1.l,  0.l,  1.l },
				{  1.l, -1.l, -1.l, -1.l },
				{  1.l, -1.l, -1.l,  1.l },
				{  1.l,  1.l,  1.l, -1.l },
				{  1.l,  1.l,  1.l,  1.l },
				{  1.l,  1.l,  0.l, -1.l },
				{  1.l,  1.l,  0.l,  1.l },
				{  1.l,  1.l, -1.l, -1.l },
				{  1.l,  1.l, -1.l,  1.l },
				{ -1.l, -1.l,  1.l, -1.l },
				{ -1.l, -1.l,  1.l,  1.l },
				{ -1.l, -1.l,  0.l, -1.l },
				{ -1.l, -1.l,  0.l,  1.l },
				{ -1.l, -1.l, -1.l, -1.l },
				{ -1.l, -1.l, -1.l,  1.l },
				{ -1.l,  1.l,  1.l, -1.l },
				{ -1.l,  1.l,  1.l,  1.l },
				{ -1.l,  1.l,  0.l, -1.l },
				{ -1.l,  1.l,  0.l,  1.l },
				{ -1.l,  1.l, -1.l, -1.l },
				{ -1.l,  1.l, -1.l,  1.l },
		};

		for (int a = 0; a < 4; a++)
		{
				output[a] = helicities[i][a];
		}
}

//Polar angle of incoming photon momentum in the x-z plane.
// [LESN] (25)
cld get_cosQ(cld egam, cld cos)
{
		cld t = t_mand(egam, cos);
		cld s = mpro * (2.l * egam + mpro); //Mandelstam variable, s

		cld E1 = (s - pow(mpro, 2.l) + t) / (2.l * mjpsi); //Energy of incoming proton
		cld E2 = (s - pow(mpro, 2.l) - pow(mjpsi, 2.l)) / (2.l * mjpsi); //Energy of recoil proton

		cld q = (mjpsi / 2.l) - (t / (2.l * mjpsi)); //Photon energy

		cld result = (pow(E1, 2.l) - pow(E2, 2.l) - pow(q, 2.l)) / (2.l * q * sqrt(pow(E2, 2.l) - pow(mpro, 2.l)));

		return result;
}

//Polar angle of incoming proton in z-x plane.
// [LESN] (37)
cld get_cosP(cld egam, cld cos)
{
		cld t = t_mand(egam, cos);
		cld s = mpro * (2.l * egam + mpro);

		cld E1 = (s - pow(mpro, 2.l) + t) / (2.l * mjpsi);
		cld E2 = (s - pow(mpro, 2.l) - pow(mjpsi, 2.l)) / (2.l * mjpsi);

		cld result = (pow(mpro, 2.l) - (E1 * E2) - (t / 2.l)) / (sqrt(pow(E1, 2.l) - pow(mpro, 2.l)) * sqrt(pow(E2, 2.l) - pow(mpro, 2.l)));
		return result;
}

// Gets appropriate proton-helicity dependent N-matrix entries
// [LESN] (32 - 35)
void get_Nentries(cld egam, cld cos, cld *helicities, cld *output)
{
		ld s1 = real(helicities[1]);
		ld s2 = real(helicities[3]);
		cld c  = sqrt((1.l + get_cosP(egam, cos)) / 2.l);
		cld s  = sqrt(1.l - pow(c, 2.l));

		if ( (s1 > 0.0l) && (s2 > 0.0l))
		{
				output[0] = s;
				output[1] = c;
				output[2] = uniti * c;
				output[3] = -s;
		}
		else if ((s1 < 0.0l) && (s2 > 0.0l))
		{
				output[0] = c;
				output[1] = -s;
				output[2] = -uniti * s;
				output[3] = -c;
		}
		else if ((s1 > 0.0l) && (s2 < 0.0l))
		{
				output[0] = -c;
				output[1] = -s;
				output[2] = uniti * s;
				output[3] = -c;
		}
		else
		{
				output[0] = s;
				output[1] = -c;
				output[2] = uniti * c;
				output[3] = s;
		}
}

// Outputs all photon and jpsi helicity-dependant  values (A, B, G, H, b and rho)
void get_ABGH(cld egam, cld cos, cld *helicities, cld *output)
{
		cld lambda = helicities[0];
		cld projM  = helicities[2];

		cld t    = t_mand(egam, cos);
		cld s    = mpro * (2.l * egam + mpro);
		cld cosP = get_cosP(egam, cos);
		cld sinP = sqrt(1.l - pow(cosP, 2.l));
		cld cosQ = get_cosQ(egam, cos);
		cld sinQ = sqrt(1.l - pow(cosQ, 2.l));

		cld E1 = (s - pow(mpro, 2.l) + t) / (2.l * mjpsi);
		cld E2 = (s - pow(mpro, 2.l) - pow(mjpsi, 2.l)) / (2.l * mjpsi);

		// Incoming Proton 3-momentum
		cld r1[3];
		r1[0] = sqrt((E1 - mpro) / (E1 + mpro)) * sinP;
		r1[1] = 0.l;
		r1[2] = sqrt((E1 - mpro) / (E1 + mpro)) * cosP;

		// Outgoing Proton 3-momentum (theta = 180 by construction)
		cld r2[3];
		r2[0] = 0.l;
		r2[1] = 0.l;
		r2[2] = -sqrt((E2 - mpro) / (E2 + mpro));

		output[0] = 1.l + (r1[0] * r2[0]) + (r1[1] * r2[1]) + (r1[2] * r2[2]);                                                              // Scalar A (12)
		output[1] = sqrt(pow((r1[0] * r2[1] - r1[1] * r2[0]), 2.l) + pow((r1[1] * r2[2] - r1[2] * r2[1]), 2.l)
		                 + pow((r1[2] * r2[0] - r1[0] * r2[2]), 2.l));
		output[2] = (-lambda / sqrt(2.l)) * (cosQ * (r1[0] + r2[0]) + uniti * lambda * (r1[1] + r2[1]) + sqrt(1.l - pow(cosQ, 2.l)) * (r1[2] + r2[2])); // Scalar G (39)

		output[3] = (-lambda * uniti / sqrt(2.l)) * (uniti * lambda * (r1[2] - r2[2]) - sqrt(1.l - pow(cosQ, 2.l)) * (r1[1] - r2[1]));      // H_x (40)
		output[4] = (-lambda * uniti / sqrt(2.l)) * (sqrt(1.l - pow(cosQ, 2.l)) * (r1[0] - r2[0]) - cosQ * (r1[2] - r2[2]));                // H_y
		output[5] = (-lambda * uniti / sqrt(2.l)) * (cosQ * (r1[1] - r2[1]) - uniti * lambda * (r1[0] - r2[0]));                            // H_z

		//b^Lambda and Rho (27 and 30 respectively)
		if (abs(real(projM)) < 0.01l)
		{
				output[6] = cosQ;       //rho
				output[7] = -lambda * sinQ / sqrt(2.l); //b^lambda
		}
		else
		{
				output[6] = (projM) * sinQ / sqrt(2.l);
				output[7] = ((cosQ - 1.l) / (2.l)) * pow(-1.l, (lambda - projM) / 2.l);
				if (abs(real(projM) - real(lambda)) < 0.001)
				{
						output[7] = 1.l + output[7];
				}
		}
}

cld background_ale(cld egam, cld cos, cld *input, int i)
{
		cld a0    = input[0];
		cld anorm = input[2];
		cld st    = input[3];
		cld ap    = input[1];
		cld b0    = input[4];

		cld s    = mpro * (2.l * egam + mpro);
		cld rs   = sqrt(s);
		cld t    = t_mand(egam, cos);
		cld sin  = sqrt(1.l - pow(cos, 2.l));
		cld tmin = t_mand(egam, 1.l);

		// Calculating the energies in the com. q is the psi momentum
		cld E1 = (s + pow(mpro, 2.l)                  ) / (2.l * rs);
		cld E2 = (s + pow(mpro, 2.l) - pow(mjpsi, 2.l)) / (2.l * rs);
		cld q  = sqrt(kallen(s, pow(mpro, 2.l), pow(mjpsi, 2.l))) / (2.l * rs);



		cld helicities[4];
		get_helicity(i, helicities);

		//polarization vectors
		cld epsilon_gamma[4], epsilonstar_psi[4], p_gamma[4];
		cld u1[4], u2bar[4];

		p_gamma[0] = rs - E1;
		p_gamma[1] = p_gamma[2] = 0.l;
		p_gamma[3] = (PARTICLE2) ? p_gamma[0] : -p_gamma[0];

		cld eta[4] = { 1.l, -1.l, -1.l, -1.l };


		// Photon in particle 2 convention

		if (PARTICLE2)
				switch ((int) real(helicities[0]) )
				{
				case 1:
						epsilon_gamma[0] = 0.l;
						epsilon_gamma[1] = -1.l/ sqrt(2.l);
						epsilon_gamma[2] = -uniti / sqrt(2.l);
						epsilon_gamma[3] = 0.l;
						break;
				case -1:
						epsilon_gamma[0] = 0.l;
						epsilon_gamma[1] = 1.l/ sqrt(2.l);
						epsilon_gamma[2] = -uniti / sqrt(2.l);
						epsilon_gamma[3] = 0.l;
						break;
				}
		else switch ((int) real(helicities[0]) )
				{
				case -1:
						epsilon_gamma[0] = 0.l;
						epsilon_gamma[1] = -1.l/ sqrt(2.l);
						epsilon_gamma[2] = -uniti / sqrt(2.l);
						epsilon_gamma[3] = 0.l;
						break;
				case 1:
						epsilon_gamma[0] = 0.l;
						epsilon_gamma[1] = 1.l/ sqrt(2.l);
						epsilon_gamma[2] = -uniti / sqrt(2.l);
						epsilon_gamma[3] = 0.l;
						break;
				}

		// Psi in particle 2 convention. This is the spinor starred, already
		if (PARTICLE2)
				switch ((int) real(helicities[2]) )
				{
				case 1:
						epsilonstar_psi[0] = 0.l;
						epsilonstar_psi[1] = -cos / sqrt(2.l);
						epsilonstar_psi[2] = uniti/ sqrt(2.l);
						epsilonstar_psi[3] =  sin / sqrt(2.l);
						break;
				case -1:
						epsilonstar_psi[0] = 0.l;
						epsilonstar_psi[1] =  cos / sqrt(2.l);
						epsilonstar_psi[2] = uniti/ sqrt(2.l);
						epsilonstar_psi[3] = -sin / sqrt(2.l);
						break;
				case 0:
						epsilonstar_psi[0] = q / mjpsi;
						epsilonstar_psi[1] = (rs - E2) / mjpsi *  sin;
						epsilonstar_psi[2] = 0.l;
						epsilonstar_psi[3] = (rs - E2) / mjpsi *  cos;
						break;
				}
		else switch ((int) real(helicities[2]) )
				{
				case -1:
						epsilonstar_psi[0] = 0.l;
						epsilonstar_psi[1] = -cos / sqrt(2.l);
						epsilonstar_psi[2] = uniti/ sqrt(2.l);
						epsilonstar_psi[3] =  sin / sqrt(2.l);
						break;
				case 1:
						epsilonstar_psi[0] = 0.l;
						epsilonstar_psi[1] =  cos / sqrt(2.l);
						epsilonstar_psi[2] = uniti/ sqrt(2.l);
						epsilonstar_psi[3] = -sin / sqrt(2.l);
						break;
				case 0:
						epsilonstar_psi[0] = -q / mjpsi;
						epsilonstar_psi[1] = (rs - E2) / mjpsi *  sin;
						epsilonstar_psi[2] = 0.l;
						epsilonstar_psi[3] = (rs - E2) / mjpsi *  cos;
						break;
				}

		if (PARTICLE2)
				switch ((int) real(helicities[1]) )
				{
				case -1:
						u1[0] = sqrt(E1 + mpro);
						u1[1] =  0.l;
						u1[2] = -sqrt(E1 - mpro);
						u1[3] = 0.l;
						break;
				case 1:
						u1[0] = 0.l;
						u1[1] =  sqrt(E1 + mpro);
						u1[2] = 0.l;
						u1[3] = sqrt(E1 - mpro);
						break;
				}
		else switch ((int) real(helicities[1]) )
				{
				case 1:
						u1[0] = sqrt(E1 + mpro);
						u1[1] =  0.l;
						u1[2] = sqrt(E1 - mpro);
						u1[3] = 0.l;
						break;
				case -1:
						u1[0] = 0.l;
						u1[1] =  sqrt(E1 + mpro);
						u1[2] = 0.l;
						u1[3] = -sqrt(E1 - mpro);
						break;
				}

		cld cshalf = sqrt((1.l + cos)/2.l);
		cld snhalf = sqrt((1.l - cos)/2.l);
		if (PARTICLE2)
				switch ((int) real(helicities[3]) )
				{
				case -1:
						u2bar[0] =  sqrt(E2 + mpro)*cshalf;
						u2bar[1] =  sqrt(E2 + mpro)*snhalf;
						u2bar[2] =  sqrt(E2 - mpro)*cshalf;
						u2bar[3] =  sqrt(E2 - mpro)*snhalf;
						break;
				case 1:
						u2bar[0] = -sqrt(E2 + mpro)*snhalf;
						u2bar[1] =  sqrt(E2 + mpro)*cshalf;
						u2bar[2] =  sqrt(E2 - mpro)*snhalf;
						u2bar[3] = -sqrt(E2 - mpro)*cshalf;
						break;
				}
		else switch ((int) real(helicities[3]) )
				{
				case 1:
						u2bar[0] =  sqrt(E2 + mpro)*cshalf;
						u2bar[1] =  sqrt(E2 + mpro)*snhalf;
						u2bar[2] = -sqrt(E2 - mpro)*cshalf;
						u2bar[3] = -sqrt(E2 - mpro)*snhalf;
						break;
				case -1:
						u2bar[0] = -sqrt(E2 + mpro)*snhalf;
						u2bar[1] =  sqrt(E2 + mpro)*cshalf;
						u2bar[2] = -sqrt(E2 - mpro)*snhalf;
						u2bar[3] =  sqrt(E2 - mpro)*cshalf;
						break;
				}



		cld numerator =  0.l;
		cld scal1 =  0.l;
		cld scal2 = 0.l;



		for (int nu = 0; nu < 4; nu++)
		{
				scal1 += epsilonstar_psi[nu] * eta[nu] * p_gamma[nu];
				scal2 += epsilonstar_psi[nu] * eta[nu] * epsilon_gamma[nu];
		}


		for (int mu=0; mu < 4; mu++)
		{
				cld spinors = 0.l;
				for (int i=0; i< 4; i++)
						for (int j=0; j< 4; j++)
								spinors += u2bar[i]*gammamatrices[mu][i][j]*u1[j];
				numerator += spinors * eta[mu] * (-epsilon_gamma[mu]*scal1 + p_gamma[mu]*scal2 );
		}

//	Uncomment this to have the old background (helicity conserving)
//	numerator = (abs(helicities[0] - helicities[2]) < 0.001l && abs(helicities[1] - helicities[3]) < 0.001l) ? s : 0.l;
		cld result = numerator * (uniti * anorm * exp(b0 * (t - tmin)) * pow(s - st, a0 + ap * t)) / s;


		return result;
}

cld background(cld egam, cld cos, cld *input, int i)
{
		cld a0    = input[0];
		cld anorm = input[2];
		cld st    = input[3];
		cld ap    = input[1];
		cld b0    = input[4];

		cld s    = mpro * (2.l * egam + mpro);
		cld t    = t_mand(egam, cos);
		cld tmin = t_mand(egam, 1.l);

		cld E1 = (s - pow(mpro, 2.l) + t) / (2.l * mjpsi);
		cld E2 = (s - pow(mpro, 2.l) - pow(mjpsi, 2.l)) / (2.l * mjpsi);
		cld q  = (mjpsi / 2.l) - (t / (2.l * mjpsi));

		cld f = sqrt((E1 + mpro) * (E2 + mpro));

		cld helicities[4];
		get_helicity(i, helicities);

		cld nmatrix[4];
		get_Nentries(egam, cos, helicities, nmatrix);

		cld ABGH[8];
		get_ABGH(egam, cos, helicities, ABGH);
		cld A       = ABGH[0];
		cld B       = ABGH[1];
		cld G       = ABGH[2];
		cld Hx      = ABGH[3];
		cld Hy      = ABGH[4];
		cld Hz      = ABGH[5];
		cld blambda = ABGH[7];
		cld rho     = ABGH[6];

		cld normalization =  f / s;

		cld result = normalization * (uniti * anorm * exp(b0 * (t - tmin)) * pow(s - st, a0 + ap * t))
		             * (nmatrix[0] * (mjpsi * blambda * A + q * rho * G)
		                + nmatrix[1] * (q * rho * Hx) + nmatrix[2] * (-uniti * mjpsi * blambda * B + q * rho * Hy)
		                + nmatrix[3] * (q * rho * Hz));
		return result;
}

//Wigner d-func for resonance in the SAME ORDER as the helicity configurations above
//Alessandro : I'm correcting this to match with the background.
// We must have d(lambda_p - lambda_gamma , lambda_p' - lambda_psi)
// i.e., we have d_(-lambda, - lambda') = (-)^(lambda-lambda') d(lambda,lambda')
void get_dfunc(cld cos, cld *input, cld *output, int i)
{
		for (int p =0; p < 2; p++)
		{
				cld jr = abs(input[8+(4*p)]);
				int naturality = ((int)(real(jr) - .5l) % 2 == 0) ? 1 : -1;
				naturality    *= (real(input[8+(4*p)]) > 0.) ? 1 : -1;
				cld a12 = input[14+p];
				cld a32 = sqrt(1.l - pow(input[14+p], 2.l));
				cld dfuncs[24];


				if (abs(jr - 1.5l) < .001)
				{
						dfuncs[0]  = a32 * pow(1.l + cos, 1.5l) / (2.l * sqrt(2.l));
						dfuncs[1]  = -a32 * (sqrt(1.5l) * sqrt(1.l - cos) * (1.l + cos)) / 2.l;
						dfuncs[2]  = -a32 * (sqrt(1.5l) * sqrt(1.l - cos) * (1.l + cos)) / 2.l;
						dfuncs[3]  = -a32 * (sqrt(1.5l) * (-1.l + cos) * sqrt(1.l + cos)) / 2.l;
						dfuncs[4]  = -a32 * (sqrt(1.5l) * (-1.l + cos) * sqrt(1.l + cos)) / 2.l;
						dfuncs[5]  = -a32 *pow(1.l - cos, 1.5l) / (2.l * sqrt(2.l));
						dfuncs[6]  = a12 * (sqrt(1.5l) * sqrt(1.l - cos) * (1.l + cos)) / 2.l;
						dfuncs[7]  = a12 * (sqrt(1.l + cos) * (-1.l + 3.l * cos)) / (2.l * sqrt(2.l));
						dfuncs[8]  = a12 * (sqrt(1.l + cos) * (-1.l + 3.l * cos)) / (2.l * sqrt(2.l));
						dfuncs[9]  = -a12 * (sqrt(2.l - 2.l * cos) * (1.l + 3.l * cos)) / 4.l;
						dfuncs[10] = -a12 * (sqrt(2.l - 2.l * cos) * (1.l + 3.l * cos)) / 4.l;
						dfuncs[11] = -a12 * (sqrt(1.5l) * (-1.l + cos) * sqrt(1.l + cos)) / 2.l;
						dfuncs[12] = -a12 * (sqrt(1.5l) * (-1.l + cos) * sqrt(1.l + cos)) / 2.l;
						dfuncs[13] = a12 * (sqrt(2.l - 2.l * cos) * (1.l + 3.l * cos)) / 4.l;
						dfuncs[14] = a12 * (sqrt(2.l - 2.l * cos) * (1.l + 3.l * cos)) / 4.l;
						dfuncs[15] = a12 * (sqrt(1.l + cos) * (-1.l + 3.l * cos)) / (2.l * sqrt(2.l));
						dfuncs[16] = a12 * (sqrt(1.l + cos) * (-1.l + 3.l * cos)) / (2.l * sqrt(2.l));
						dfuncs[17] = -a12 * (sqrt(1.5l) * sqrt(1.l - cos) * (1.l + cos)) / 2.l;
						dfuncs[18] = a32 * pow(1.l - cos, 1.5l) / (2.l * sqrt(2.l));
						dfuncs[19] = -a32 * (sqrt(1.5l) * (-1.l + cos) * sqrt(1.l + cos)) / 2.l;
						dfuncs[20] = -a32 * (sqrt(1.5l) * (-1.l + cos) * sqrt(1.l + cos)) / 2.l;
						dfuncs[21] = a32 * (sqrt(1.5l) * sqrt(1.l - cos) * (1.l + cos)) / 2.l;
						dfuncs[22] = a32 * (sqrt(1.5l) * sqrt(1.l - cos) * (1.l + cos)) / 2.l;
						dfuncs[23] = a32 * pow(1.l + cos, 1.5l) / (2.l * sqrt(2.l));
				}
				else
				{
						dfuncs[0]  = a32 * (pow(1.l + cos, 1.5l) * (-3.l + 5.l * cos)) / (4.l * sqrt(2.l));
						dfuncs[1]  = -a32 * (sqrt(1.l - cos) * (1.l + cos) * (-1.l + 5.l * cos)) / 4.l;
						dfuncs[2]  = -a32 * (sqrt(1.l - cos) * (1.l + cos) * (-1.l + 5.l * cos)) / 4.l;
						dfuncs[3]  = -a32 * ((-1.l + cos) * sqrt(1.l + cos) * (1.l + 5.l * cos)) / 4.l;
						dfuncs[4]  = -a32 * ((-1.l + cos) * sqrt(1.l + cos) * (1.l + 5.l * cos)) / 4.l;
						dfuncs[5]  = -a32 * (pow(1.l - cos, 1.5l) * (3.l + 5.l * cos)) / (4.l * sqrt(2.l));
						dfuncs[6]  = a12 * (sqrt(1.l - cos) * (1.l + cos) * (-1.l + 5.l * cos)) / 4.l;
						dfuncs[7]  = a12 * (sqrt(1.l + cos) * (-1.l - 2.l * cos + 5.l * pow(cos, 2.l))) / (2.l * sqrt(2.l));
						dfuncs[8]  = a12 * (sqrt(1.l + cos) * (-1.l - 2.l * cos + 5.l * pow(cos, 2.l))) / (2.l * sqrt(2.l));
						dfuncs[9]  = a12 * (sqrt(2.l - 2.l * cos) * (1.l - 2.l * cos - 5.l * pow(cos, 2.l))) / 4.l;
						dfuncs[10] = a12 * (sqrt(2.l - 2.l * cos) * (1.l - 2.l * cos - 5.l * pow(cos, 2.l))) / 4.l;
						dfuncs[11] = -a12 * ((-1.l + cos) * sqrt(1.l + cos) * (1.l + 5.l * cos)) / 4.l;
						dfuncs[12] = -a12 * ((-1.l + cos) * sqrt(1.l + cos) * (1.l + 5.l * cos)) / 4.l;
						dfuncs[13] = a12 * (sqrt(2.l - 2.l * cos) * (-1.l + 2.l * cos + 5.l * pow(cos, 2.l))) / 4.l;
						dfuncs[14] = a12 * (sqrt(2.l - 2.l * cos) * (-1.l + 2.l * cos + 5.l * pow(cos, 2.l))) / 4.l;
						dfuncs[15] = a12 * (sqrt(1.l + cos) * (-1.l - 2.l * cos + 5.l * pow(cos, 2.l))) / (2.l * sqrt(2.l));
						dfuncs[16] = a12 * (sqrt(1.l + cos) * (-1.l - 2.l * cos + 5.l * pow(cos, 2.l))) / (2.l * sqrt(2.l));
						dfuncs[17] = -a12 * (sqrt(1.l - cos) * (1.l + cos) * (-1.l + 5.l * cos)) / 4.l;
						dfuncs[18] = a32 * (pow(1.l - cos, 1.5l) * (3.l + 5.l * cos)) / (4.l * sqrt(2.l));
						dfuncs[19] = -a32 * ((-1.l + cos) * sqrt(1.l + cos) * (1.l + 5.l * cos)) / 4.l;
						dfuncs[20] = -a32 * ((-1.l + cos) * sqrt(1.l + cos) * (1.l + 5.l * cos)) / 4.l;
						dfuncs[21] = a32 * (sqrt(1.l - cos) * (1.l + cos) * (-1.l + 5.l * cos)) / 4.l;
						dfuncs[22] = a32 * (sqrt(1.l - cos) * (1.l + cos) * (-1.l + 5.l * cos)) / 4.l;
						dfuncs[23] = a32 * (pow(1.l + cos, 1.5l) * (-3.l + 5.l * cos)) / (4.l * sqrt(2.l));
				}


				cld helicities[4];
				get_helicity(i, helicities);
				if (naturality < 0)
				{
						if (real(helicities[0]) < 0.l) dfuncs[i] *= -1.l;
						if (real(helicities[2] - .5l*helicities[3]) < 0.l) dfuncs[i] *= -1.l;
				}

				if (PARTICLE2 == 1)
				{ output[p] = dfuncs[i];}
				else
				{

						int expon = (int) real(helicities[1]/2.l - helicities[0] - helicities[3]/2.l + helicities[2]);
						expon = expon % 2 == 0 ? 1 : -1;
						output[p] = dfuncs[i]*(ld)expon;
				}
		}

}
//Pc(4450) Resonance contribution
// [BLIN] (2)
cld resonance(cld egam, cld cos, cld *input, int i)
{
		//[0] is appropriate dfunc for Pc(4450); [1] is for Pc(4380)
		cld dfunc10q[2];
		get_dfunc(cos, input, dfunc10q, i);
		cld result = 0.l;
		for (int p = 0; p < 2; p++)
		{
				cld angmom, pt;
				cld st    = input[3];
				cld xBrn   = input[5+(4*p)];
				cld xMassr = input[6+(4*p)];
				cld xGamr  = input[7+(4*p)];
				cld jrpr   = input[8+(4*p)];
				cld jr     = abs(input[8+(4*p)]);
				// 3/2-
				if      (abs(real(jrpr) + 1.5l) < .001l) { angmom = 0.l; pt = 2.l/3.l; }
				// 3/2+
				else if (abs(real(jrpr) - 1.5l) < .001l) { angmom = 1.l; pt = 3.l/5.l; }
				// 5/2+
				else if (abs(real(jrpr) - 2.5l) < .001l) { angmom = 1.l; pt = 3.l/5.l; }
				// 5/2-
				else if (abs(real(jrpr) + 2.5l) < .001l) { angmom = 2.l; pt = 1.l/3.l; }

				//Mandelstam Variables
				cld s    = mpro * (2.l * egam + mpro);
				cld t    = t_mand(egam, cos);

				cld W = sqrt(s);
				cld pfbar = sqrt(kallen(xMassr*xMassr, mjpsi*mjpsi, mpro*mpro))/2.l/xMassr;
				cld pf    = sqrt(kallen(W*W, mjpsi*mjpsi, mpro*mpro))/2.l/W;
				cld pibar = (xMassr*xMassr - mpro*mpro)/2.l/xMassr;
				cld pi    = (W*W           - mpro*mpro)/2.l/W;

				cld g = sqrt((xBrn * xGamr * xMassr * xMassr * 8.l * Pi * (2.l*jr + 1.l))/(6.l * pf));
				cld Gamma_gamma = xBrn * xGamr * pow(xFjpsi/mjpsi,2.l) * pow(pibar/pfbar, 2.l * angmom + 1.l) * pt; // 4 pi alpha removed
				cld a = sqrt(Gamma_gamma * Pi * xMassr * (2.l * jr + 1.l)/(pibar*pibar *2.l* mpro));
				cld amp = W / xMassr * pibar  * sqrt(8.l * mpro * xMassr / pi) * a * g * dfunc10q[p];// 4 pi alpha removed

				//Breit Wigner
				cld pentaquark = amp / ((uniti*xGamr*xMassr - pow(xMassr,2.l) + s));

				// Threshold Factor
				pentaquark *= pow((s - st)/s, 1.5l);
				pentaquark /= pow((xMassr*xMassr - st)/xMassr/xMassr, 1.5l);

				result += pentaquark;
		}
		return result;
}

//Produce ith Helicity Amplitude (incoherent sum of both contributions)
cld AmpFunc(cld egam, cld cos, cld *input, int i)
{
		//cout << background(egam,cos,input, i) << " " << resonance(egam,cos,input,i) << endl;
		cld result = BACKGROUNDCHOICE(egam,cos,input, i) + resonance(egam,cos,input,i);
		return result;
}


/*=======================================================================================
 *  AUXILIARY FUNCTIONS FOR OBSERVABLES (UNSMEARED)
 *=========================================================================================*/

//DIFFERENTIAL CROSS SECTION (unnormalized! just sum over all 24 amplutides squared)
cld DXSFunc(cld *amps)
{
		cld sum = 0.l;
		for (int i = 0; i<24; i++)
		{
				sum = sum + (amps[i] * conj(amps[i]));
		}
		return sum;

}

//KLL asymmetry
void KLL(cld *amps, cld * output)
{
		cld sigmapp = 0.l;
		cld sigmapm = 0.l;
		for (int i = 0; i < 6; i++)
		{
				sigmapp = sigmapp + (amps[(2*i)+1] * conj(amps[(2*i)+1]));
				sigmapm = sigmapm + (amps[2*i] * conj(amps[2*i]));
		}
		output[0] = (sigmapp - sigmapm);
		output[1] = (sigmapp + sigmapm);
}

//ALL asymmetry
void ALL(cld *amps, cld * output)
{
		cld sigmapp = 0.l;
		cld sigmapm = 0.l;
		for (int i = 0; i < 6; i++)
		{
				sigmapp = sigmapp + (amps[(i)+6] * conj(amps[(i)+6]));
				sigmapm = sigmapm + (amps[i] * conj(amps[i]));
		}
		output[0] = (sigmapp - sigmapm);
		output[1] = (sigmapp + sigmapm);
}

// THREE SDMES
cld rho1p(cld *amps)
{
		cld sdme1=0.l;
		for (int i = 0; i < 8; i++)
		{
				sdme1 = sdme1 + (amps[iter[i]] * conj(amps[iter[i]])); //Square of amps with jpsi helicity M = +1
		}
		return sdme1;
}

cld rho10(cld *amps)
{
		cld sdme2= 0.l;
		for (int i = 0; i < 8; i++)
		{
				sdme2 = sdme2 + (amps[iter[i]] * conj(amps[iter[i]+2])); // (M = +1) * conj(M = 0)
		}
		return sdme2;
}

cld rho1m(cld *amps)
{
		cld sdme3=0.l;
		for (int i = 0; i < 8; i++)
		{
				sdme3 = sdme3 + (amps[iter[i]] * conj(amps[iter[i]+4])); // (M = +1) * conj(M = -1)
		}
		return sdme3;
}

/*=======================================================================================
 *  AUXILIARY FUNCTIONS FOR SMEARING OBSERVABLES
 *=========================================================================================*/

//Numerical Integration
void DSG20R(ld& xlow, ld& xhigh, ld& n, ld *xd, ld& np)
{
		ld y[CONST2] = { .9931285991, .9639719272, .9122344282, .8391169718,
				 .7463319064,  .6360536807, .5108670019, .3737060887,
				 .2277858511, .0765265211 };
		ld xn, dint, delt, orig, dorig;
		int i, i1, i2, j, j1, j2;

		np   = 20.l * n;
		xn   = n;
		dint = xhigh - xlow;
		dint = dint / xn;
		delt = dint / 2.;
		orig = xlow - delt;
		i1   = -20;
		for (i = 0; i < int(n); i++)
		{
				orig  = orig + dint;
				dorig = orig + orig;
				i1    = i1 + 20;
				i2    = i1 + 19;

				for (j = 0; j < CONST2; j++)
				{
						j1     = i1 + j;
						j2     = i2 - j;
						xd[j1] = orig - delt * y[j];
						xd[j2] = dorig - xd[j1];
				}
		}
}


void DRG20_complex(ld& xlow, ld& xhigh, ld& n, cld *fd_c, cld& int_result)
{
		ld w[CONST2] = { .0176140071, .0406014298, .0626720483, .0832767415,
				 .1019301198,  .1181945319, .1316886384, .1420961093,
				 .1491729864, .1527533871 };
		cld cr;
		int i, i1, i2, j, j1, j2;

		cr = 0.;
		i1 = -20;
		for (i = 0; i < int(n); i++)
		{
				i1 = i1 + 20;
				i2 = i1 + 19;
				for (j = 0; j < CONST2; j++)
				{
						j1 = i1 + j;
						j2 = i2 - j;
						cr = cr + w[j] * (fd_c[j1] + fd_c[j2]);
				}
		}
		int_result = cr * .5l * (xhigh - xlow) / n;
}

cld get_cosCM(cld egam, cld t)
{
		cld esqr = pow(egam,2.l);
		cld mpsqr = pow(mpro,2.l);
		cld mjsqr = pow(mjpsi,2.l);

		cld result = (2.l*esqr*mpro - egam*mjsqr + 2.l*egam*t - mpro*mjsqr + mpro*t)/(egam*sqrt(4.l*esqr*mpsqr - 4.l*mpro*mjsqr*(egam+mpro) + pow(mjpsi, 4.l)));
		return result;
}

void get_cos_bounds(cld egam, cld* bounds)
{
		bounds[1] = get_cosCM(egam, -0.1l);
		bounds[0] = get_cosCM(egam, -1.05l);
		return;
}

extern "C"
void forfortran_(double *egam, double *cos, double *input_double, double *output)
{
		cld input[11];
		for (int i=0; i < 11; i++)
		{
				input[i] = input_double[i];
		}
		ld total = 0.l;
		for (int i = 0; i < 24; i++)
		{
				total += pow(abs(BACKGROUNDCHOICE(*egam, *cos, input, i) + resonance (*egam, *cos, input, i)), 2.l);
		}
		// 6084.375 is (hc)^2 / 64 pi(kinematics) * 4 pi(e^2->alpha) / 4(polarizations) in nb GeV^2
		*output = (6084.375l * real(alpha)) / pow((*egam)*real(mpro), 2.l)  * total;
		return;
}

void get_inputs(cld jp, cld * input)
{
		string filename;

		switch ((int) (real(jp)*2.l)) {
		case 200: {filename = "./src/background-inputs/test.txt"; cout << "testing!" << endl; break;}
		case -3: {filename = "./src/background-inputs/32m-0.txt"; break;}
		case 3: filename = "./src/background-inputs/32p-0.txt"; break;
		case -5: filename = "./src/background-inputs/52m-0.txt"; break;
		case 5: filename = "./src/background-inputs/52p-0.txt"; break;
		default: cout << "ERROR! J^P doesn't match any input file." << endl; exit(1);
		}
		ifstream inputfile;
		inputfile.open(filename.c_str());
		for (int i = 0; i < 13; i++)
		{
				inputfile >> input[i];
		}
		inputfile.close();
}
