  #include <iostream>
  #include <fstream>
  #include <math.h>
  #include <cmath>
  #include <complex>
  #include <iomanip>
  #include <cstdlib>
  #include <stdio.h>
  #include <string.h>
  #include <sstream>
  #include <stdlib.h>

using namespace std;
typedef complex<long double>   cld;
typedef long double ld;

#ifndef HEADER
#define HEADER

  #define ACCSMEAR    100  // for energy-integration accuracy to obtain functions smeared with a Gaussian
  #define ACCTOT      1    // for angle-integration accuracy to obtain total cross section
  #define CONST1      2000 // for integration routines - do not change
  #define CONST2      10   // for integration routines - do not change
  #define PARTICLE2   0   // if 0 the fermions have particle2 conventions, if 1 they have particle1
  #define BACKGROUNDCHOICE background

/*Global Constants*/
const cld Pi     = 3.14159265359l;
const cld alpha  = 1.l / 137.l;
const cld mpro   = 0.9383l;   // Proton mass
const cld mjpsi  = 3.097l;    // J/psi mass
const cld xFjpsi = 0.278l;    // J/psi decay constant
const cld uniti(0.l, 1.l);    // Imaginary unit i
const cld unitr(1., 0.);

const int iter[8] = {0,1, 6, 7, 12, 13, 18, 19};  //Iterator for SDME summations - do not change

const cld gammamatrices[6][4][4] = {
//gamma0
		{ { 1.l, 0.l, 0.l, 0.l }, { 0.l, 1.l, 0.l, 0.l }, { 0.l, 0.l, -1.l, 0.l }, { 0.l, 0.l, 0.l, -1.l } },
//gamma1
		{ { 0.l, 0.l, 0.l, 1.l }, { 0.l, 0.l, 1.l, 0.l }, { 0.l, -1.l, 0.l, 0.l }, { -1.l, 0.l, 0.l, 0.l } },
//gamma2
		{ { 0.l, 0.l, 0.l, -uniti }, { 0.l, 0.l, uniti, 0.l }, { 0.l,  uniti, 0.l, 0.l }, { -uniti, 0.l, 0.l, 0.l } },
//gamma3
		{ { 0.l, 0.l, 1.l, 0.l }, { 0.l, 0.l, 0.l, -1.l }, { -1.l, 0.l, 0.l, 0.l }, { 0.l, 1.l, 0.l, 0.l } },
//gamma4 = gamma0
		{ { 1.l, 0.l, 0.l, 0.l }, { 0.l, 1.l, 0.l, 0.l }, { 0.l, 0.l, -1.l, 0.l }, { 0.l, 0.l, 0.l, -1.l } },
//gamma5
		{ { 0.l, 0.l, 1.l, 0.l }, { 0.l, 0.l, 0.l, 1.l }, { 1.l, 0.l, 0.l, 0.l }, { 0.l, 1.l, 0.l, 0.l } }
};

/*=======================================================================================
 *  MAIN
 *=========================================================================================*/

void ObservablesParam(cld egam, cld cos, cld *input, cld *output);

/*=======================================================================================
 *  AUXILIARY FUNCTIONS FOR HELICITY AMPLITUDES
 *=========================================================================================*/

// Mandelstam variable, t, in terms of lab frame photon energy.
cld t_mand(cld egam, cld cos);

void get_config(double * config);
double kallen(double x, double y, double z);
cld kallen(cld x, cld y, cld z);


//Scattering angle of the recoil proton in the t-channel LAB frame
cld get_cosLAB(cld egam, cld cos);

// 24 Helicity Amplitudes are needed to describe jpsi production. Only 12 ( lambda_gamma = +1)
// is used for the background
// Helicity configuration for Photon [0], Incoming Proton [1], Jpsi [2], Outgoing Proton [3]
void get_helicity(int i, cld *output);


//Polar angle of incoming photon momentum in the x-z plane.
// [LESN] (25)
cld get_cosQ(cld egam, cld cos);

//Polar angle of incoming proton in z-x plane.
// [LESN] (37)
cld get_cosP(cld egam, cld cos);

// Gets appropriate proton-helicity dependent N-matrix entries
// [LESN] (32 - 35)
void get_Nentries(cld egam, cld cos, cld *helicities, cld *output);

// Outputs all photon and jpsi helicity-dependant  values (A, B, G, H, b and rho)
void get_ABGH(cld egam, cld cos, cld *helicities, cld *output);



cld background(cld egam, cld cos, cld *input, int i);
cld background_ale(cld egam, cld cos, cld *input, int i);

void get_dfunc(cld cos, cld *input, cld*output, int i);
cld resonance(cld egam, cld cos, cld *input, int i);

cld AmpFunc(cld egam, cld cos, cld *input, int i);

/*=======================================================================================
 *  AUXILIARY FUNCTIONS FOR OBSERVABLES (UNSMEARED)
 *=========================================================================================*/

//DIFFERENTIAL CROSS SECTION (unnormalized! just sum over all 24 amplutides squared)
cld DXSFunc(cld *amps);

//KLL asymmetry
void KLL(cld *amps, cld * output);

//ALL asymmetry
void ALL(cld *amps, cld * output);

// THREE SDMES
cld rho1p(cld *amps);

cld rho10(cld *amps);

cld rho1m(cld *amps);

/*=======================================================================================
 *  AUXILIARY FUNCTIONS FOR SMEARING OBSERVABLES
 *=========================================================================================*/

//Numerical Integration
void DSG20R(ld& xlow, ld& xhigh, ld& n, ld *xd, ld& np);

void get_cos_bounds(cld egam, cld* bounds);

void get_inputs(cld jp, cld * input);

void DRG20_complex(ld& xlow, ld& xhigh, ld& n, cld *fd_c, cld& int_result);

int mc(int n, double jpp, double BR, double photocouplings, string file);


extern "C"
void forfortran_(double *egam, double *cos, double *input_double, double *output);

#endif
