#include <TStyle.h>
#include <TCanvas.h>
#include <TProfile.h>
#include <TH1D.h>
#include <TH3F.h>
#include <TF1.h>
#include <TChain.h>
#include <TString.h>
#include <vector>
#include <string>
#include "TROOT.h"
#include "TFrame.h"
#include "TLatex.h"
#include "TFile.h"
#include "TMath.h"
//#include "lhcbStyle.h"
#include "TGenPhaseSpace.h"
#include "TRandom.h"
#include <iostream>
#include <fstream>

//----------- Using ROOT

Double_t frac_brem(Double_t *, Double_t *);
double calc_brem(Double_t, Double_t, Double_t, Double_t );
Double_t chisq2sigma(Double_t, Int_t );


void tests()
{

		//tests
		cout<<"fraction (8.2-11): "<< calc_brem(8.2,11.,0.10,11.) <<endl;
		cout<<"Hall C [Meziani] expected (~2.3%)"<<endl;
		cout<<""<<endl;

		cout<<"chisq 10. and dof=7... "<<endl;
		cout<<"n. sigma is: "<< chisq2sigma(10.5,7)<<endl;

}

double calc_brem(Double_t lowE, Double_t highE, double X0, double Eb)
{
		//X0 radiation lenght (%)
		//Eb electron beam energy

		Double_t res;
		TF1*f = new TF1("f",frac_brem,0.,11.,2); //2: # of pars
		f->SetParameter(0,X0); // see Meziani
		f->SetParameter(1,Eb);

		res = f->Integral(lowE,highE);

		return res;

}

Double_t frac_brem(Double_t *x, Double_t *par)
{
		Double_t res;
		res= (1./x[0])*(TMath::Power(1-x[0]/par[1],4./3.*par[0])-TMath::Exp(-7./9.*par[0]))/(7./9.+4./3.*TMath::Log(1.-x[0]/par[1]));
		return res; //N.B. GeV^{-1}
		//par[0] is R.L. in %
		//par[1] is endpoint E electron beam

		//FIT if you are using ROOT:
		//
}

Double_t chisq2sigma(Double_t chisq, Int_t dof)
{

		Double_t nsigma, tmp_nsigma;
		Double_t prob, tmp_prob;
		Double_t prov_point;

		//sorry I have been lazy here. I'll change it later...

		for(double tmp_nsigma=0.; tmp_nsigma<=10; tmp_nsigma+=0.001) //0.01 accuracy in the random search
		{
				tmp_prob = ROOT::Math::normal_cdf(tmp_nsigma,1.,0)-ROOT::Math::normal_cdf_c(tmp_nsigma, 1.,0);
				prov_point = TMath::ChisquareQuantile(tmp_prob,dof);
				if(TMath::Abs(prov_point-chisq)/chisq<0.01)
				{
						nsigma = tmp_nsigma;
						break;
				}
		}

		return nsigma;

}
