#include "TGenPhaseSpace.h"
#include "TLorentzVector.h"
#include "TH2.h"
#include "TH1.h"
#include "TROOT.h"
#include "TRandom3.h"
#include "TMath.h"
#define PI TMath::Pi()
#include "TFile.h"
#include "TTree.h"
#include "TComplex.h"
#include "amp.h"
#include "TF1.h"
#include "TF2.h"
#include "Math/WrappedTF1.h"
#include "Math/GSLIntegrator.h"
#include "Math/Integrator.h"
#include "Math/GaussIntegrator.h"
#include <cmath>
#include <string>


cld input[17];
cld input_B[17];
double rad2deg = (180./3.14159);
double e_electron_lab, X0, intensity, rhoL, days;
double frac_brem(double *x, double *par);
double dsigmadcostheta(double *x, double *par);
double dsigmadcostheta2D(double *x, double *par);
const double mpsi = real(mjpsi), mp = real(mpro);
const double e_thr = mpsi + mpsi*mpsi/mp/2.;
cld WignerD_j1(int lambda_1, int lambda_2, double phi, double costheta)
{
		cld ret = exp(-((long double)lambda_1 * phi) * uniti);
		if (lambda_1 == 1 && lambda_2 == 1) ret *= (1. + costheta)/2.;
		else if (lambda_1 == 1 && lambda_2 == 0) ret *= -TMath::Sqrt((1. - costheta*costheta)/2.);
		else if (lambda_1 == 1 && lambda_2 == -1) ret *=  (1. - costheta)/2.;
		else if (lambda_1 == 0 && lambda_2 == 1) ret *= TMath::Sqrt((1. - costheta*costheta)/2.);
		else if (lambda_1 == 0 && lambda_2 == 0) ret *= costheta;
		else if (lambda_1 == 0 && lambda_2 == -1) ret *= -TMath::Sqrt((1. - costheta*costheta)/2.);
		else if (lambda_1 == -1 && lambda_2 == 1) ret *=  (1. - costheta)/2.;
		else if (lambda_1 == -1 && lambda_2 == 0) ret *= TMath::Sqrt((1. - costheta*costheta)/2.);
		else if (lambda_1 == -1 && lambda_2 == -1) ret *= (1. + costheta)/2.;
		else ret = 0.l;
		return ret;
}

double sgn(double x)
{
	if (x > 0) return 1.;
	else if (x < 0) return -1.;
	else return 0.;
}

double WignerD_rotation(int lambda_1, int lambda_2, double costheta)
{
		//signs for sintheta < 0 as in this case
		double ret;
		if (lambda_1 == 1 && lambda_2 == 1) ret = TMath::Sqrt((1. + costheta)/2.);
		else if (lambda_1 == 1 && lambda_2 == -1) ret = TMath::Sqrt((1. - costheta)/2.);
		else if (lambda_1 == -1 && lambda_2 == -1) ret = TMath::Sqrt((1. + costheta)/2.);
		else if (lambda_1 == -1 && lambda_2 == 1) ret = -TMath::Sqrt((1. - costheta)/2.);
		else ret = 0.l;
		return ret;
}


int mc2(long int n, double spinparity, double BRfrac, double photocouplings, string filename, bool fixed,
	bool quick, bool saveall, int bins, double smear, double min_bin, double max_bin)
{

		long int nn = n; int seed = 0;

		if (fixed) printf("Generating %ld events with seed %d at e_gamma(LAB) = %lf GeV... \n", nn, seed, e_electron_lab);
		else printf("Generating %ld events with seed %d at e_electron(LAB) = %lf GeV... \n", nn, seed, e_electron_lab);
		TRandom3 *rnd = new TRandom3(seed);
		filename = filename + "-" + to_string(spinparity) + "-" + to_string(BRfrac) + "-" + to_string(photocouplings) + ".root";
		TFile *file = new TFile(filename.c_str(), "RECREATE");
		TTree *t= new TTree("ntuple", "ntuple");
		double ep_px, ep_py, ep_pz, ep_E;
		double em_px, em_py, em_pz, em_E;
		double prec_px, prec_py, prec_pz, prec_E;
		double pgamma_px, pgamma_py, pgamma_pz, pgamma_E;
		double ptarg_px, ptarg_py, ptarg_pz, ptarg_E;
		double weight_SIGNAL, weight_NOWIG, costheta_ep, costheta_psi, phi_ep, phi_psi;
		int lambda_gamma, lambda_ptarg, deltalambda, lambda_prec;
		double costhetawigner;
		double energy_bins[101];
		long int nn_bins[101];
		bool IN_DETECTOR;

	//	double weight_BACK;

		double e_gamma_lab;
		if (fixed) e_gamma_lab = e_electron_lab;

		for (int i = 0; i <= bins; i++)
		{
			energy_bins[i] =  min_bin + (max_bin - min_bin) * i / (double)bins;
			nn_bins[i] = 0;
		}

		//Grabs INPUTS
		get_inputs(spinparity,input);
		input[14] = photocouplings;
		input[15] = .7071;

		input[5] = BRfrac;
		input[9] = BRfrac;

		get_inputs(spinparity,input_B);
		input_B[5] = 0.;
		input_B[9] = 0.;
		input_B[14] = 0.7071;

		t->Branch("ep_px", &ep_px);
		t->Branch("ep_py", &ep_py);
		t->Branch("ep_pz", &ep_pz);
		t->Branch("ep_E", &ep_E);
		t->Branch("em_px", &em_px);
		t->Branch("em_py", &em_py);
		t->Branch("em_pz", &em_pz);
		t->Branch("em_E", &em_E);
		t->Branch("prec_px", &prec_px);
		t->Branch("prec_py", &prec_py);
		t->Branch("prec_pz", &prec_pz);
		t->Branch("prec_E", &prec_E);
	  // t->Branch("pgamma_px", &pgamma_px);
	  // t->Branch("pgamma_py", &pgamma_py);
	  // t->Branch("pgamma_pz", &pgamma_pz);
	  // t->Branch("pgamma_E", &pgamma_E);
//  t->Branch("ptarg_px", &ptarg_px);
//  t->Branch("ptarg_py", &ptarg_py);
//  t->Branch("ptarg_pz", &ptarg_pz);
//  t->Branch("ptarg_E", &ptarg_E);
		t->Branch("lambda_gamma", &lambda_gamma);
		t->Branch("lambda_ptarg", &lambda_ptarg);
		t->Branch("lambda_prec", &lambda_prec);
		t->Branch("deltalambda", &deltalambda);
		t->Branch("phi_psi", &phi_psi);
		t->Branch("costheta_psi", &costheta_psi);
		t->Branch("phi_ep", &phi_ep);
		t->Branch("costheta_ep", &costheta_ep);
		t->Branch("costheta_wigner", &costhetawigner);

		t->Branch("weight_SIGNAL", &weight_SIGNAL);
		t->Branch("weight_NOWIG", &weight_NOWIG);
		t->Branch("e_electron_lab", &e_electron_lab);
		t->Branch("e_gamma_lab", &e_gamma_lab);

		t->Branch("spin_parity", &spinparity);
		t->Branch("BR", &BRfrac);
		t->Branch("IN_DETECTOR", &IN_DETECTOR);

		TH1F *th1_phi_psi = new TH1F("th1_phi_psi", ":phi:radians", 100, 0., 2*PI);
		TH1F *th1_phi_ep = new TH1F("th1_phi_ep", ":phi:radians", 100, 0., 2*PI);
		TH1F *th1_costheta_psi = new TH1F("th1_costheta_psi", "costheta", 100, -1., 1.);
		TH1F *th1_costheta_ep = new TH1F("th1_costheta_ep", "costheta", 100, -1., 1.);

		double mp2 = mp*mp, mpsi2 = mpsi*mpsi;
		double weight_IN[bins], weight2_IN[bins], weight_OUT[bins], ALL[bins], KLL[bins];
		for (int i = 0; i < bins; i++) weight_IN[i] = weight_OUT[i] =  weight2_IN[i] = ALL[i] = KLL[i] = 0.;


		for (long int n=0; n<nn; n++)
		{
				if (!fixed) e_gamma_lab = e_electron_lab - rnd->Rndm() * (e_electron_lab - e_thr);

				phi_psi = 2*PI*rnd->Rndm();
				costheta_psi = 2*rnd->Rndm() - 1.;
				phi_ep = 2*PI*rnd->Rndm();
				costheta_ep = 2*rnd->Rndm() - 1.;
				double s = 2.*e_gamma_lab*mp + mp2;
				double rs = TMath::Sqrt(s);

				TLorentzVector p_ep(0., 0., mpsi/2., mpsi/2.);
				TLorentzVector p_em(0., 0.,-mpsi/2., mpsi/2.);
				TLorentzVector p_psi(0., 0., 0., mpsi);

				// everything in the psi rest frame, in the right direction
				p_ep.RotateY(TMath::ACos(costheta_ep));
				p_em.RotateY(TMath::ACos(costheta_ep));
				p_ep.RotateZ(phi_ep);
				p_em.RotateZ(phi_ep);

				// we boost to the psi p rest frame, in the z direction, then rotate everything to the psi flight direction
				TVector3 boost_psi(0., 0., TMath::Sqrt(kallen(s, mpsi2, mp2))/(s + mpsi2 - mp2));

				p_ep.Boost(boost_psi);
				p_em.Boost(boost_psi);
				p_psi.Boost(boost_psi);
				p_ep.RotateY(TMath::ACos(costheta_psi));
				p_em.RotateY(TMath::ACos(costheta_psi));
				p_psi.RotateY(TMath::ACos(costheta_psi));
				p_ep.RotateZ(phi_psi);
				p_em.RotateZ(phi_psi);
				p_psi.RotateZ(phi_psi);

				TLorentzVector p_prec(-p_psi.X(), -p_psi.Y(), -p_psi.Z(), rs - p_psi.E());
				TLorentzVector p_ptarg(0., 0., (mp2 - s)/2./rs, (s + mp2)/2./rs);
				TLorentzVector p_gamma(0., 0., (s - mp2)/2./rs, (s - mp2)/2./rs);

				// we boost everything in the lab frame
				TVector3 boost_lab(0., 0., (s - mp2)/(s + mp2));

				p_ep.Boost(boost_lab);
				p_em.Boost(boost_lab);
				p_psi.Boost(boost_lab);
				p_ptarg.Boost(boost_lab);
				p_gamma.Boost(boost_lab);
				p_prec.Boost(boost_lab);

				ep_px = p_ep.X();  ep_py = p_ep.Y();  ep_pz = p_ep.Z();  ep_E = p_ep.E();
				em_px = p_em.X();  em_py = p_em.Y();  em_pz = p_em.Z();  em_E = p_em.E();
				prec_px = p_prec.X();  prec_py = p_prec.Y();  prec_pz = p_prec.Z();  prec_E = p_prec.E();
				pgamma_px = p_gamma.X();  pgamma_py = p_gamma.Y();  pgamma_pz = p_gamma.Z();  pgamma_E = p_gamma.E();
				ptarg_px = p_ptarg.X();  ptarg_py = p_ptarg.Y();  ptarg_pz = p_ptarg.Z();  ptarg_E = p_ptarg.E();

				// check if in the ficucial region of SBS
				// if proton has recoil angle theta = 17 deg. into hadron calorimeter
				// Acceptance calculated from semiwidth +/- 50 cm in the X-Z plane and +/- 100 cm in the Y, with distance 385 cm from production vertex
				// weird use of angles -- theta is from -pi to pi, phi is from -pi/2 to pi/2. It works though
				IN_DETECTOR = false;
				double p_arm = 17., e_arm = 22.;

				if(p_prec.P() > 1. && p_em.P() > 1. && p_ep.E() > 2.)
					if( abs((atan2(prec_px, prec_pz)*rad2deg -  p_arm)) < 7.4 && abs(atan2(prec_py, sqrt(prec_px*prec_px + prec_pz*prec_pz)))*rad2deg < 14.6 )
					{
							//and if either positron or electron in electron arm with other in hadron arm
							// 22 deg. opposite the hadron arm
							// Acceptance calculated from semiwidth +/- 50 cm in the X-Z plane and +/- 150 cm in the Y, with distance 400 cm from production vertex
							if ((abs((atan2(ep_px, ep_pz)*rad2deg + e_arm)) < 7.1 && abs(atan2(ep_py, sqrt(ep_px*ep_px + ep_pz*ep_pz)))*rad2deg < 20.6)
							    &&
							    ((abs(atan2(em_px, em_pz)*rad2deg -  p_arm)) < 7.4 && abs(atan2(em_py, sqrt(em_px*em_px + em_pz*em_pz)))*rad2deg < 14.6))
							{
									IN_DETECTOR = true;
							}

							else if ((abs((atan2(em_px, em_pz)*rad2deg + e_arm)) < 7.1 && abs(atan2(em_py, sqrt(em_px*em_px + em_pz*em_pz)))*rad2deg < 20.6)
							         &&
							         (abs(atan2(ep_px, ep_pz)*rad2deg -  p_arm) < 7.4 && abs(atan2(ep_py, sqrt(ep_px*ep_px + ep_pz*ep_pz)))*rad2deg < 14.6))
							{
									IN_DETECTOR = true;
							}
					}

				// Uncomment to remove the acceptance check
				//IN_DETECTOR = true;

				double e_gamma_lab_smear = e_gamma_lab;
				if (smear > 0.) e_gamma_lab_smear += rnd->Gaus(0., smear);

				int i = bins*(e_gamma_lab_smear - min_bin)/(max_bin - min_bin);
				if (i < 0 || i >= bins) continue;

				nn_bins[i]++;
				if (quick && !IN_DETECTOR) continue;
	//			cout << e_gamma_lab << endl;

				cld helicities[4], amplitude_SIGNAL = 0., amplitude_NOWIG = 0., amplitude_BACKGROUND = 0.;
				lambda_gamma = 1 - 2*((int)(2*rnd->Rndm()));
				lambda_ptarg = 1 - 2*((int)(2*rnd->Rndm()));
				lambda_prec = 1 - 2*((int)(2*rnd->Rndm()));
				deltalambda = 1 - 2*((int)(2*rnd->Rndm()));


				cld amplitude_UP = 0., amplitude_DOWN = 0.; // we need both in the cm frame
				for (int i=0; i<24; i++)
				{
						get_helicity(i, helicities);
						if (abs(helicities[0] - (ld)lambda_gamma) > 0.001 || abs(helicities[1] - (ld)lambda_ptarg) > 0.001) continue;

						// Computes weight for both signal with BR and photocouplings and background only for comparison with kinematics all the same.

						if (abs(helicities[3] - (ld)1.) < 0.001)
							amplitude_UP += AmpFunc((cld)e_gamma_lab, (cld) costheta_psi, input, i)*WignerD_j1((int)real(helicities[2]), deltalambda, phi_ep, costheta_ep );
						else if (abs(helicities[3] + (ld)1.) < 0.001)
							amplitude_DOWN += AmpFunc((cld)e_gamma_lab, (cld) costheta_psi, input, i)*WignerD_j1((int)real(helicities[2]), deltalambda, phi_ep, costheta_ep );

				}

				//double sinthetawigner = -TMath::Sqrt(1. - costheta_psi*costheta_psi)*(s - mp2) / 2. / rs / p_prec.P();
				// sinthetawigner is always < 0
				costhetawigner = ((s + mp2) * TMath::Sqrt(kallen(s, mp2, mpsi2)) - costheta_psi * (s + mp2 - mpsi2) * (s - mp2))
				                        / ( 4*s*mp* p_prec.P());
				//												printf("Wigner: %lf %lf %lf\n", costhetawigner, sinthetawigner, pow(costhetawigner,2)+pow(sinthetawigner,2));
				amplitude_SIGNAL = amplitude_UP*(ld)WignerD_rotation(lambda_prec, 1, costhetawigner) + amplitude_DOWN*(ld)WignerD_rotation(lambda_prec, -1, costhetawigner);
				amplitude_NOWIG = (lambda_prec > 0) ? amplitude_UP : amplitude_DOWN;

				double parbeam[2] = { X0, e_electron_lab };
				double xbeam[2] = { 0., e_gamma_lab };
				double weight_BEAM = (fixed) ? 1. : frac_brem(xbeam,parbeam);
				weight_SIGNAL = real(amplitude_SIGNAL*conj(amplitude_SIGNAL)) * weight_BEAM;

				if (IN_DETECTOR == true)
				{
						weight_IN[i] += weight_SIGNAL;
						weight2_IN[i] += weight_SIGNAL*weight_SIGNAL;


						ALL[i] += weight_SIGNAL * ((lambda_gamma == lambda_ptarg) ? 1. : -1.);
						KLL[i] += weight_SIGNAL * ((lambda_gamma == lambda_prec) ? 1. : -1.);

						t->Fill();

						th1_phi_psi->Fill(phi_psi, weight_SIGNAL);
						th1_phi_ep->Fill(phi_ep, weight_SIGNAL);
						th1_costheta_psi->Fill(costheta_psi, weight_SIGNAL);
						th1_costheta_ep->Fill(costheta_ep, weight_SIGNAL);
				}
				else
				{
						weight_OUT[i] += weight_SIGNAL;
						if (saveall) t->Fill();
				}
		}


		t->Write();
		th1_phi_psi->Write();
		th1_phi_ep->Write();
		th1_costheta_psi->Write();
		th1_costheta_ep->Write();

		cout << endl;
		cout << "Generated " << t->GetEntries() << " events in the fiducial region of SBS..." << endl;
//		cout << "WEIGHTS " << weight_IN << " " << weight_OUT << endl;


		char outname[256];
		sprintf(outname,"./txt/output_sp_%1.3f_BR_%1.3f_PC_%1.3f_RL_%1.3f_nbins_%d_Emin_%1.3f_Emax_%1.3f_sm_%1.3f.txt", spinparity, BRfrac, photocouplings,X0,bins,min_bin,max_bin,smear);
		//sprintf(outname,"./txt/output_sp_%1.3f_BR_%1.3f_PC_%1.3f_RL_%1.3f.txt", spinparity, BRfrac, photocouplings,X0);

		FILE *fi = fopen(outname,"w");

		if (bins == 1)
		{

			if (quick)
			{
					printf("Acceptance (quick) : %.6lf, ALL : %.6lf +- %.6lf, KLL : %.6lf\n",
								(double)t->GetEntries() / (double)nn, ALL[0]/weight_IN[0], sqrt(weight2_IN[0])/weight_IN[0], KLL[0]/weight_IN[0]);
			}
			else
			{
					printf("Acceptance total : %.6lf, ALL : %.6lf, KLL : %.6lf\n", weight_IN[0] / (weight_IN[0] + weight_OUT[0]), ALL[0]/weight_IN[0], KLL[0]/weight_IN[0]);
			}

			if (fixed)
			{
				TF1 f("disgmadcostheta", dsigmadcostheta, -1., 1.);
				ROOT::Math::WrappedTF1 wf1(f);

				ROOT::Math::GaussIntegrator ig;

				// Set parameters of the integration
				ig.SetFunction(wf1);
				ig.SetRelTolerance(0.001);
				double xsect = ig.Integral(-1.,1.);
				// printf("Done.\n\n");
				printf("xsect @ e_gamma(LAB) = %lf GeV : %lf nb x BR(psi -> ee)\n", e_gamma_lab, xsect);

				fprintf(fi, "ENRGY\tACCPT\tALL\tKLL\tXSECT\n");
				fprintf(fi, "%lf\t%lf\t%lf\t%lf\t%lf\n", e_gamma_lab,
								quick ? ((double)t->GetEntries() / (double)nn) : ( weight_IN[0] / (weight_IN[0] + weight_OUT[0])),
								ALL[0]/weight_IN[0], KLL[0]/weight_IN[0], xsect
							);
				cout << endl;
			}
			else
			{
				TF2 f2("disgmadcostheta2D", dsigmadcostheta2D, -1., 1., -1.,1., 2);
				f2.SetParameter(0, X0); // Radiation Length X0
				f2.SetParameter(1, e_electron_lab); // Beam Energy in the lab frame
		//cout << real(mjpsi)+real(mjpsi*mjpsi/mpro)/2. << endl;
				//ROOT::Math::WrappedTF2 wf2(f);
				double xsect = f2.Integral(-1.,1.,e_thr, e_electron_lab);
				printf("xsect X f(E) integrated : %lf nb x BR(psi -> ee)\n", xsect);
				fprintf(fi, "EN_LOW\tEN_HI\tACCPT\tALL\tKLL\tXSECT\n");
				fprintf(fi, "%lf\t%lf\t%lf\t%lf\t%lf\t%lf\n", e_thr, e_electron_lab,
								quick ? ((double)t->GetEntries() / (double)nn) : ( weight_IN[0] / (weight_IN[0] + weight_OUT[0])),
								ALL[0]/weight_IN[0], KLL[0]/weight_IN[0], xsect
							);


				//cout << "xsect X f(E) integrated: " << f2.Integral(-1.,1.,mpsi+mpsi*mpsi/mp/2., e_electron_lab) << endl;
			}

		}
		else
		{
			double ALL_total = 0., KLL_total = 0.;
			TF2 f2("disgmadcostheta2D", dsigmadcostheta2D, -1., 1., -1.,1., 2);
			f2.SetParameter(0, X0); // Radiation Length X0
			f2.SetParameter(1, e_electron_lab); // Beam Energy in the lab frame

			fprintf(fi, "EN_LOW\tEN_HI\tACCPT\tALL\tKLL\tXSECT\n");

			double wi = 0., wo = 0., w2i = 0.;
			for (int i = 0; i < bins; i++) { wi += weight_IN[i]; w2i += weight2_IN[i]; wo += weight_OUT[i]; }

			for (int i = 0; i < bins; i++)
			{
					ALL_total += ALL[i]; KLL_total += KLL[i];
					//cout << real(mjpsi)+real(mjpsi*mjpsi/mpro)/2. << endl;
					//ROOT::Math::WrappedTF2 wf2(f);
					double xsect = f2.Integral(-1.,1.,energy_bins[i], energy_bins[i+1]);
					//cout << "xsect X f(E) integrated: " << f2.Integral(-1.,1.,mpsi+mpsi*mpsi/mp/2., e_electron_lab) << endl;
					if (quick)
							printf("Acceptance (quick) in bin %.2lf - %.2lf : %.6lf (%.2le +- %.2le we), ALL : %.2le +- %.2le, KLL : %.2le, xsect : %.6lf nb X BR(psi ->ee)\n", energy_bins[i], energy_bins[i+1],
//										(double)t->GetEntries("e_gamma_lab > " << energy_bins[i] << " && e_gamma_lab < " << energy_bins[i]) / (double)nn_bins[i], 0.);
											(double)t->GetEntries(Form("e_gamma_lab > %lf && e_gamma_lab < %lf", energy_bins[i], energy_bins[i+1]))/ (double)nn_bins[i],
											weight_IN[i], sqrt(weight2_IN[i]), ALL[i]/weight_IN[i], sqrt(weight2_IN[i])/weight_IN[i], KLL[i]/weight_IN[i], xsect);
					else
							printf("Acceptance in bin %.2lf - %.2lf : %.2le (%.2le +- %.2le weighted events), ALL : %.2le +- %.2le, KLL : %.2le, xsect : %.6lf nb X BR(psi ->ee)\n", energy_bins[i], energy_bins[i+1],
										weight_IN[i] / (weight_IN[i] + weight_OUT[i]), 	weight_IN[i], sqrt(weight2_IN[i]), ALL[i]/weight_IN[i], sqrt(weight2_IN[i])/weight_IN[i], KLL[i]/weight_IN[i], xsect);

					fprintf(fi, "%lf\t%lf\t%lf\t%lf\t%lf\t%lf\n", energy_bins[i], energy_bins[i+1],
									quick ? ((double)t->GetEntries(Form("e_gamma_lab > %lf && e_gamma_lab < %lf", energy_bins[i], energy_bins[i+1]))/ (double)nn_bins[i]) : ( weight_IN[i] / (weight_IN[i] + weight_OUT[i])),
									ALL[i]/weight_IN[i], KLL[i]/weight_IN[i], xsect
									);


			}

			if (quick)
					printf("Acceptance (quick) total : %.6lf (%8ld events), ALL : %.6lf +- %.6lf, KLL : %.6lf\n",
								(double)t->GetEntries() / (double)nn, nn, ALL_total/wi, sqrt(w2i)/wi, KLL_total/wi);
			else
					printf("Acceptance total : %.6lf (%8ld events), ALL : %.6lf +- %.6lf, KLL : %.6lf\n",
								wi / (wi + wo), nn, ALL_total/wi,  sqrt(w2i)/wi, KLL_total/wi);

			double xsect = f2.Integral(-1.,1.,e_thr, e_electron_lab);
			printf("xsect X f(E) integrated : %lf nb x BR(psi -> ee)\n", xsect);
			fprintf(fi, "%lf\t%lf\t%lf\t%lf\t%lf\t%lf\n", e_thr, e_electron_lab,
							quick ? ((double)t->GetEntries()/ (double)nn) : ( wi / (wi + wo)),
							ALL_total/wi, KLL_total/wi, xsect
							);


		}
		cout << endl;

		file->Close();
		fclose(fi);

		return 0;
}

double frac_brem(double *x, double *par)
{
   return (1./x[1])*(TMath::Power(1-x[1]/par[1],4./3.*par[0])-TMath::Exp(-7./9.*par[0]))/(7./9.+4./3.*TMath::Log(1.-x[1]/par[1]));
   //return res; //N.B. GeV^{-1}
   //par[0] is R.L. in %
   //par[1] is endpoint E electron beam

   //FIT if you are using ROOT:
   //
}
double dsigmadcostheta2D(double *x, double *par)
{
	double z = x[0];
	double en = x[1];
	if (en > par[1]) return 0.;
	ld total = 0.l;
	for (int i = 0; i < 24; i++)
	{
			total += pow(abs(background((cld)en, (cld) z, input, i) + resonance ((cld)en, (cld) z, input, i)), 2.l);
	}
	// 6084.375 for nb/GeV^2
	double s = real(mpro) * (2. * en + real(mpro));

	double pf = sqrt((s - pow(real(mjpsi + mpro),2.))*(s - pow(real(mjpsi - mpro),2.)));

	total *= 6084.375l * real(alpha) * pf / (s * real(mpro) * en);

	total *= frac_brem(x,par);
	return total;
}


double dsigmadcostheta(double *x, double *par)
{
		double z = x[0];
		double e_gamma_lab = e_electron_lab;
		ld total = 0.l;
		for (int i = 0; i < 24; i++)
		{
				total += pow(abs(background((cld)e_gamma_lab, (cld) z, input, i) + resonance ((cld)e_gamma_lab, (cld) z, input, i)), 2.l);
		}
		// 6084.375 for nb/GeV^2
		double s = real(mpro) * (2. * e_gamma_lab + real(mpro));

		double pf = sqrt((s - pow(real(mjpsi + mpro),2.))*(s - pow(real(mjpsi - mpro),2.)));

		total *= 6084.375l * real(alpha) * pf / (s * real(mpro) * e_gamma_lab);

		return total;
}

int main(int argc, char** argv)
{
	e_electron_lab = 11.;
	long int n = 1000000;
	double photo = 1./sqrt(2.);
	double br = .02;
	bool fixed = false;
	bool quick = false;
	bool saveall = false;
	double spinparity = 1.5;
	int bins = 1;
	double smear = 0.;
	double min_bin = e_thr, max_bin = e_electron_lab;

	if (argc == 1)
	{
		cout << "Usage: mc2 [ -n EVENTS ] [ -e  ENERGY ] [ -s SPINPARITY ] [ -b BR ] [ -p PHOTOC ] [ -x X0 ] [ -q ] [ -bins BIN ] [ -sm SMEAR ] [ -min MIN ] [ -max MAX ]" << endl;
		cout << "Either ENERGY (fixed photon energy) or X0 can be used" << endl;
		cout << "BIN is the number of bins for the photon (only with X0)" << endl;
		cout << "MIN and MAX can only be used with X0" << endl;
		cout << "SMEAR can only be used with X0 and BINS > 1" << endl;
		return 0;
	}
	for (int ii = 1; ii < argc; ii++)
	{
		if (strcmp(argv[ii],"-n")==0) { n = atol(argv[ii+1]); if (n <= 0) return 0; }
		if (strcmp(argv[ii],"-e")==0) { e_electron_lab = atof(argv[ii+1]); fixed = true; }
		if (strcmp(argv[ii],"-s")==0)
		{
			spinparity = atof(argv[ii+1]);
			if (abs(spinparity) != 1.5 && abs(spinparity) != 2.5) { cout << "ERROR: spinparity " << spinparity << " incorrect" << endl; return 1; }
		}
		if (strcmp(argv[ii],"-b")==0)
		{
			br = atof(argv[ii+1]);
			if (br < 0. || br > 1.) { cout << "ERROR: branching ratio " << br << " incorrect" << endl; return 1; }
		}
		if (strcmp(argv[ii],"-p")==0)
		{
			photo = atof(argv[ii+1]);
			if (photo < 0. || photo > 1.) { cout << "ERROR: photocoupling ratio " << photo << " incorrect" << endl; return 1; }
		}
		if (strcmp(argv[ii],"-x")==0)
		{
			X0 = atof(argv[ii+1]);
			if (fixed) { cout << "ERROR: either -e (fixed energy) or -x (radiation length) can be given" << endl; return 1; }
		}
		if (strcmp(argv[ii],"-bins")==0)
		{
			bins = atoi(argv[ii+1]);
			if (bins < 1 || bins > 100) { cout << "ERROR: invalid bin number" << endl; return 1; }
			if (fixed) { cout << "ERROR: BINS cannot be used with fixed energy" << endl; return 1; }
		}

		if (strcmp(argv[ii],"-sm")==0)
		{
			smear = atof(argv[ii+1]);
			if (smear < 0.) { cout << "ERROR: invalid smearing" << endl; return 1; }
			if (fixed || bins == 1) { cout << "ERROR: SMEARING cannot be used with fixed energy or with one bin" << endl; return 1; }
		}

		if (strcmp(argv[ii],"-q")==0)
		{
			quick = true;
		}

		if (strcmp(argv[ii],"-saveall")==0)
		{
			saveall = true;
		}

		if (strcmp(argv[ii],"-min")==0)
		{
			if (fixed) { cout << "ERROR: MIN and MAX cannot be used with fixed energy" << endl; return 1; }
			if (min_bin < e_thr)  { printf("ERROR: MIN cannot be < %.2lf (threshold)", e_thr); return 1; }
			min_bin = atof(argv[ii+1]);
		}
		if (strcmp(argv[ii],"-max")==0)
		{
			if (fixed) { cout << "ERROR: MIN and MAX cannot be used with fixed energy" << endl; return 1; }
			if (max_bin > e_electron_lab)  { printf("ERROR: MIN cannot be > %.2lf (beam energy)", e_electron_lab); return 1; }
			max_bin = atof(argv[ii+1]);
		}

	}

	// cout << "KLL setup:" << endl;
	// X0 = 0.06; rhoL = 2.3e23;
	// intensity = 3.1e14;
	// days = 9 * 86400;
	// cout << "X0 = " << X0 << ", intensity = " << intensity << ", rhoL = " << rhoL << ", days = " << days << endl;
	mc2(n, spinparity, br, photo, "./output/output", fixed, quick, saveall, bins, smear, min_bin, max_bin);
	return 0;

}

// void mc()
// {
//              main(1, NULL);
//              return;
// }
