#include <iostream>
#include <fstream>
#include "amp.h"

int main(int argc, char** argv)
{
        cld cos = 1.l;
        cld stepsize = 0.01l;
        cld maxe = 22.0l;
        cld smear = 0.l;
        cld jp = -1.5l;
        cld r1 = -1l;
        cld r2 = -1l;
        cld photo1 = .7071l;
        int plot = 0;

        for (int ii=0; ii<argc; ii++)
        {
                if (strcmp(argv[ii],"-c")==0) cos = atof(argv[ii+1]);
                if (strcmp(argv[ii],"-b")==0) stepsize = atof(argv[ii+1]);
                if (strcmp(argv[ii],"-m")==0) maxe = atof(argv[ii+1]);
                if (strcmp(argv[ii],"-s")==0) smear = atof(argv[ii+1]);
                if (strcmp(argv[ii],"-j")==0) jp = atof(argv[ii+1]);
                if (strcmp(argv[ii],"-r_n")==0) r1 = atof(argv[ii+1]);
                if (strcmp(argv[ii],"-r_w")==0) r2 = atof(argv[ii+1]);
                if (strcmp(argv[ii],"-p")==0) photo1 = atof(argv[ii+1]);
                if (strcmp(argv[ii],"-plot")==0) plot = 1;
        }

        //Grabs INPUTS
        cld input[17];
        get_inputs(jp, input);
        input[16] = smear;
        // input[14] = photo1;

        if (real(r1) >= 0)
        {
                input[5] = r1;
        }
        if (real(r2) >= 0)
        {
                input[9] = r2;
        }
        else
        {
                cout << "Input BR for Pc(4380)!" << endl; exit(1);
        }

        cld amplitudes[24];
        cld ratio[3] = {0.l, .7071l, 1.l};
        cld zero[5];
        cld equal[5];
        cld one[5];
        cld background[5];
        cld egam;

        ofstream photo;
        photo.open("./output/photo-wide.dat");
        photo << "#Fixed Cosine of Scattering Angle = " << real(cos) << endl;
        photo << left << setw(30) << "#Background, A_norm:" << setw(20) << real(input[2]) << endl;
        photo << left << setw(30) << "#Pc(4380) spin/parity: " << setw(20) << real(input[12]) << endl;
        photo << left << setw(30) << "#     BR:" << setw(20) << real(input[9]) << endl;
        photo << left << setw(30) << "#Pc(4450) spin/parity: " << setw(20) << real(input[8]) << endl;
        photo << left << setw(30) << "#     BR:" << setw(20) << real(input[5]) << endl;
        photo << left << setw(30) << "#     Photocouplings:" << setw(20) << real(input[14]) << endl;
        photo << left << setw(30) << "#Smearing, sigma_s: " << setw(5) << real(input[9]) << "GeV" << endl;
        photo << left << setw(10) << "# zero -> [A1/2 = 0]    " << setw(10) << "equal -> [A1/2 = A3/2]    " << setw(10) << "one -> [A3/2 = 0]   " << endl;
        photo << left << setw(30) << "" << endl;
        photo << left << setw(30) << "#Egamma" << setw(30)
              << "KLL (zero)" << setw(30) << "ALL (zero)" << setw(30) << "KLL (equal)" << setw(30) << "ALL (equal)" <<setw(30)
              << "KLL (one)" << setw(30) << "ALL (one)" << endl;

        for (int a = 0; a < int(real((maxe-8.3l)/stepsize))+1; a++)
        {
                egam =  8.3l + cld(a)*stepsize;
                input[15] = ratio[0];
                input[14] = ratio[0];
                ObservablesParam(egam, cos, input, zero);
                input[15] = ratio[1];
                input[14] = ratio[1];
                ObservablesParam(egam, cos, input, equal);
                input[15] = ratio[2];
                input[14] = ratio[2];
                ObservablesParam(egam, cos, input, one);

                photo << left << setw(30) << real(egam)  << setw(30)
                      << real(zero[1]) << setw(30) << real(zero[2])
                      << setw(30) << real(equal[1]) << setw(30)
                      << real(equal[2]) << setw(30) << real(one[1])
                      << setw(30) << real(one[2]) << endl;
        }
        photo.close();

        if (plot ==1)
        {
                system("mkdir -p ./output/plots && gnuplot ./src/gnuplot/quick-plot-photo2.p");
        }
        return 0;
}
