#include <iostream>
#include <fstream>
#include "amp.h"

int main(int argc, char** argv)
{
        // Default Values
        cld cos = 1.l;
        cld stepsize = 0.01l;
        cld maxe = 22.0l;
        cld smear = 0.l;
        cld photo1 = .7071l;
        cld photo2 = .7071l;
        cld r1 = -1l;
        cld r2 = -1l;
        int plot = 0;

        for (int ii=0; ii<argc; ii++)
        {
                if (strcmp(argv[ii],"-c")==0) cos = atof(argv[ii+1]);
                if (strcmp(argv[ii],"-b")==0) stepsize = atof(argv[ii+1]);
                if (strcmp(argv[ii],"-m")==0) maxe = atof(argv[ii+1]);
                if (strcmp(argv[ii],"-s")==0) smear = atof(argv[ii+1]);
                if (strcmp(argv[ii],"-p_n")==0) photo1 = atof(argv[ii+1]);
                if (strcmp(argv[ii],"-p_w")==0) photo2 = atof(argv[ii+1]);
                if (strcmp(argv[ii],"-r_n")==0) r1 = atof(argv[ii+1]);
                if (strcmp(argv[ii],"-r_w")==0) r2 = atof(argv[ii+1]);
                if (strcmp(argv[ii],"-plot")==0) plot = 1;
        }

        cld amplitudes[24];
        cld spin[4] = {-1.5l, 1.5l, -2.5l, 2.5l};
        cld s32m[5];
        cld s32p[5];
        cld s52m[5];
        cld s52p[5];
        cld egam;

        cld input[4][17];
        for (int i; i < 4; i++)
        {
                get_inputs(spin[i],input[i]);
                if (real(r1) >= 0)
                {
                        input[i][5] = r1;
                }
                if (real(r2) >= 0)
                {
                        input[i][9] = r2;
                }

                input[i][16] = smear;
                input[i][14] = photo1;
                input[i][15] = photo2;
        }

        ofstream jp;
        jp.open("./output/spin-parity.dat");
        jp << "#Fixed Cosine of Scattering Angle = " << real(cos) << endl;
        jp << left << setw(30) << "#Smearing, sigma_s: " << setw(5) << real(smear) << "GeV" << endl;
        jp << left << setw(30) << "Spin-Parity Below refers to Pc(4450), other Pc state is opposite parity and spin" << endl;
        jp << left << setw(30) << "#Egamma" << setw(30)
           << "KLL (3/2-)" << setw(30) << "ALL (3/2-)" << setw(30) << "KLL (3/2+)" << setw(30) << "ALL (3/2+)" <<setw(30)
           << "KLL (5/2-)" << setw(30) << "ALL (5/2-)" << setw(30) << "KLL (5/2+)" << setw(30) << "ALL (5/2+)" << endl;

        for (int a = 0; a < int(real((maxe-8.3l)/stepsize))+1; a++)
        {
                egam =  8.3l + cld(a)*stepsize;
                ObservablesParam(egam, cos, input[0], s32m);
                ObservablesParam(egam, cos, input[1], s32p);
                ObservablesParam(egam, cos, input[2], s52m);
                ObservablesParam(egam, cos, input[3], s52p);

                jp << left << setw(30) << real(egam)  << setw(30)
                   << real(s32m[1]) << setw(30) << real(s32m[2])
                   << setw(30) << real(s32p[1]) << setw(30)
                   << real(s32p[2]) << setw(30) << real(s52m[1])
                   << setw(30) << real(s52m[2]) << setw(30)
                   << real(s52p[1]) << setw(30) << real(s52p[2]) << endl;
        }
        jp.close();

        if (plot ==1)
        {
                system("mkdir -p ./output/plots && gnuplot ./src/gnuplot/quick-plot-spinparity.p");
        }
        return 0;
}
